package com.sodius.logging;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This class provides the log method of the application.
 * 
 * @author &Eacute;ric V&eacute;pa
 * @version $Id: MyLogger.java 888 2007-08-13 15:48:16Z evepa $
 * @since 1.5
 */
public class MyLogger {

	/**
	 * Execution state (0 if no problems occurs during the execution, 1
	 * otherwise).
	 */
	private static int status = 0;

	/**
	 * The unique instance of the class (singleton).
	 */
	private static MyLogger myLogger = null;

	/**
	 * The loggers used by the application.
	 */
	private static Map<Class<?>, Logger> loggers = null;

	/**
	 * Initializes the default logger of the application.
	 */
	private MyLogger() {
		loggers = new HashMap<Class<?>, Logger>();
		PropertyConfigurator.configure(this.getClass().getResource(
				"log4j.properties"));
	}

	/**
	 * Returns the logger of the application.
	 * 
	 * @return the logger of the application
	 */
	private static Logger getLogger(Class<?> className) {
		if (myLogger == null)
			myLogger = new MyLogger();

		if (!loggers.containsKey(className))
			loggers.put(className, Logger.getLogger(className));

		return (Logger) loggers.get(className);
	}

	/**
	 * Log a message object with the DEBUG Level for a given class.
	 * 
	 * @param className
	 *            the caller class
	 * @param message
	 *            the message object to log
	 */
	public static void debug(Class<?> className, Object message) {
		getLogger(className).debug(message + " (" + className.getName() + ")");
	}

	/**
	 * Log a message object with the INFO Level for a given class.
	 * 
	 * @param className
	 *            the caller class
	 * @param message
	 *            the message object to log
	 */
	public static void info(Class<?> className, Object message) {
		getLogger(className).info(message + " (" + className.getName() + ")");
	}

	/**
	 * Log a message object with the WARN Level for a given class.
	 * 
	 * @param className
	 *            the caller class
	 * @param message
	 *            the message object to log
	 */
	public static void warn(Class<?> className, Object message) {
		status = 1;
		getLogger(className).warn(message + " (" + className.getName() + ")");
	}

	/**
	 * Log a message object with the ERROR Level for a given class.
	 * 
	 * @param className
	 *            the caller class
	 * @param message
	 *            the message object to log
	 */
	public static void error(Class<?> className, Object message) {
		status = 1;
		getLogger(className).error(message + " (" + className.getName() + ")");
	}

	/**
	 * Log a message object with the FATAL Level for a given class.
	 * 
	 * @param className
	 *            the caller class
	 * @param message
	 *            the message object to log
	 */
	public static void fatal(Class<?> className, Object message) {
		status = 1;
		getLogger(className).fatal(message + " (" + className.getName() + ")");
	}

	/**
	 * Returns the status of the execution (0 if no problems occurs during the
	 * execution, 1 otherwise).
	 * 
	 * @return the status code of the execution
	 */
	public static int getStatus() {
		return status;
	}

}
